@@ -2,7 +2,8 @@ MixNRMI2 <-
 function (x, probs = c(0.025, 0.5, 0.975), Alpha = 1, Beta = 0, 
     Gama = 0.4, distr.k = 1, distr.py0 = 1, distr.pz0 = 2, mu.pz0 = 3, 
     sigma.pz0 = sqrt(10), delta = 4, kappa = 2, Delta = 2, Meps = 0.01, 
-    Nx = 100, Nit = 500, Pbi = 0.1, epsilon = NULL, printtime = TRUE) 
+    Nx = 150, Nit = 1500, Pbi = 0.1, epsilon = NULL, printtime = TRUE, 
+    extras = FALSE) 
 {
     if (is.null(distr.k)) 
         stop("Argument distr.k is NULL. Should be provided. See help for details.")
@@ -11,12 +12,9 @@ function (x, probs = c(0.025, 0.5, 0.975), Alpha = 1, Beta = 0,
     tInit <- proc.time()
     n <- length(x)
     y <- x
-    for (i in 1:(n/2)) {
-        y[i] <- mean(x[1:(n/2)])
-    }
-    for (i in (n/2):n) {
-        y[i] <- mean(x[(n/2):n])
-    }
+    xsort = sort(x)
+    y[seq(n/2)] <- mean(xsort[seq(n/2)])
+    y[-seq(n/2)] <- mean(xsort[-seq(n/2)])
     z <- rep(1, n)
     u <- 1
     if (is.null(epsilon)) 
@@ -27,10 +25,17 @@ function (x, probs = c(0.025, 0.5, 0.975), Alpha = 1, Beta = 0,
     R <- seq(Nit)
     U <- seq(Nit)
     Nmt <- seq(Nit)
+    Allocs <- vector(mode = "list", length = Nit)
+    if (extras) {
+        means <- vector(mode = "list", length = Nit)
+        sigmas <- vector(mode = "list", length = Nit)
+        weights <- vector(mode = "list", length = Nit)
+        Js <- vector(mode = "list", length = Nit)
+    }
     mu.py0 = mean(x)
     sigma.py0 = sd(x)
     for (j in seq(Nit)) {
-        if (floor(j/100) == ceiling(j/100)) 
+        if (floor(j/500) == ceiling(j/500)) 
             cat("MCMC iteration", j, "of", Nit, "\n")
         tt <- comp2(y, z)
         ystar <- tt$ystar
@@ -38,6 +43,7 @@ function (x, probs = c(0.025, 0.5, 0.975), Alpha = 1, Beta = 0,
         nstar <- tt$nstar
         rstar <- tt$rstar
         idx <- tt$idx
+        Allocs[[max(1, j - 1)]] <- idx
         if (Gama != 0) 
             u <- gs3(u, n = n, r = rstar, alpha = Alpha, beta = Beta, 
                 gama = Gama, delta = Delta)
@@ -68,7 +74,15 @@ function (x, probs = c(0.025, 0.5, 0.975), Alpha = 1, Beta = 0,
         R[j] <- rstar
         U[j] <- u
         Nmt[j] <- Nm
+        if (extras) {
+            means[[j]] <- Tauy
+            sigmas[[j]] <- Tauz
+            weights[[j]] <- J/sum(J)
+            Js[[j]] <- J
+        }
     }
+    tt <- comp2(y, z)
+    Allocs[[Nit]] <- tt$idx
     biseq <- seq(floor(Pbi * Nit))
     Fxx <- Fxx[, -biseq]
     qx <- as.data.frame(t(apply(Fxx, 1, quantile, probs = probs)))
@@ -76,11 +90,27 @@ function (x, probs = c(0.025, 0.5, 0.975), Alpha = 1, Beta = 0,
     qx <- cbind(mean = apply(Fxx, 1, mean), qx)
     R <- R[-biseq]
     U <- U[-biseq]
-    cpo <- 1/apply(1/fx, 1, mean)
+    Allocs <- Allocs[-biseq]
+    if (extras) {
+        means <- means[-biseq]
+        sigmas <- sigmas[-biseq]
+        weights <- weights[-biseq]
+        Js <- Js[-biseq]
+    }
+    cpo <- 1/apply(1/fx[, -biseq], 1, mean)
     if (printtime) {
         cat(" >>> Total processing time (sec.):\n")
         print(procTime <- proc.time() - tInit)
     }
-    return(list(xx = xx, qx = qx, cpo = cpo, R = R, U = U, Nm = Nmt, 
-        Nx = Nx, Nit = Nit, Pbi = Pbi, procTime = procTime))
+    if (extras) {
+        return(list(xx = xx, qx = qx, cpo = cpo, R = R, U = U, 
+            Allocs = Allocs, means = means, sigmas = sigmas, 
+            weights = weights, Js = Js, Nm = Nmt, Nx = Nx, Nit = Nit, 
+            Pbi = Pbi, procTime = procTime))
+    }
+    else {
+        return(list(xx = xx, qx = qx, cpo = cpo, R = R, U = U, 
+            Allocs = Allocs, Nm = Nmt, Nx = Nx, Nit = Nit, Pbi = Pbi, 
+            procTime = procTime))
+    }
 }
